@@ -31,24 +31,49 @@ static void zenity_text_size_allocate (GtkWidget *widget, GtkAllocation *allocat
 static void
 zenity_msg_construct_question_dialog (GtkWidget *dialog, ZenityMsgData *msg_data, ZenityData *data)
 {
+
+
   GtkWidget *cancel_button, *ok_button;
   
-  cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_NO, GTK_RESPONSE_CANCEL);
-  ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_YES, GTK_RESPONSE_OK);
+  cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_No"), GTK_RESPONSE_CANCEL);
+  ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Yes"), GTK_RESPONSE_OK);
 
   gtk_widget_grab_focus (msg_data->default_cancel ? cancel_button : ok_button);
 
   if (data->cancel_label) {
     gtk_button_set_label (GTK_BUTTON (cancel_button), data->cancel_label);
-    gtk_button_set_image (GTK_BUTTON (cancel_button), 
-                          gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
   }
 
   if (data->ok_label) {
     gtk_button_set_label (GTK_BUTTON (ok_button), data->ok_label);
-    gtk_button_set_image (GTK_BUTTON (ok_button), 
-                          gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON));
   }
+  
+
+}
+
+static void
+zenity_label_widget_clipboard_selection(GtkWidget *widget)
+{
+  /* Workaround hotfix for suspected toolkit issue:
+     since focus change of the dialog's focussed widget (text)
+     somehow currently chooses to destroy
+     a pre-existing (read: foreign, user-initiated) X11 primary selection
+     (via gtk_label_select_region() -> ...
+     -> gtk_clipboard_set_contents()/gtk_clipboard_clear()),
+     we need to ensure
+     that the widget does have its gtk-label-select-on-focus property off,
+     in order to avoid having the label become selected automatically
+     and thereby having pre-existing clipboard content nullified.
+     Side note: this selection issue only applies to widgets
+     which have both
+                  <property name="can_focus">True</property>
+                  <property name="selectable">True</property>
+     .
+   */
+  g_object_set(gtk_widget_get_settings (widget),
+               "gtk-label-select-on-focus",
+               FALSE,
+               NULL);
 }
 
 void 
@@ -70,6 +95,7 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
       break;
 
     case ZENITY_MSG_QUESTION:
+    case ZENITY_MSG_SWITCH:
       builder = zenity_util_load_ui_file ("zenity_question_dialog", NULL);
       dialog = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_question_dialog"));
       text = gtk_builder_get_object (builder, "zenity_question_text");
@@ -103,6 +129,14 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
       break;	
   }
 
+  if (data->extra_label) {
+    gint i=0;
+    while(data->extra_label[i]!=NULL){
+      gtk_dialog_add_button (GTK_DIALOG (dialog), data->extra_label[i], i);
+      i++;
+    }
+  }
+
   if (builder == NULL) {
     data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
@@ -119,27 +153,29 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
   if (ok_button) {
     if (data->ok_label) {
       gtk_button_set_label (GTK_BUTTON (ok_button), data->ok_label);
-      gtk_button_set_image (GTK_BUTTON (ok_button),
-                            gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON));
     }
   }
 
   switch (msg_data->mode) {
     case ZENITY_MSG_WARNING:
-      zenity_util_set_window_icon_from_stock (dialog, data->window_icon, GTK_STOCK_DIALOG_WARNING);
+      zenity_util_set_window_icon_from_icon_name (dialog, data->window_icon, "dialog-warning");
       break;
 
     case ZENITY_MSG_QUESTION:
-      zenity_util_set_window_icon_from_stock (dialog, data->window_icon, GTK_STOCK_DIALOG_QUESTION);
+      zenity_util_set_window_icon_from_icon_name (dialog, data->window_icon, "dialog-question");
       zenity_msg_construct_question_dialog (dialog, msg_data, data);
       break;
       
+    case ZENITY_MSG_SWITCH:
+      zenity_util_set_window_icon_from_icon_name (dialog, data->window_icon, "dialog-question");
+      break;
+      
     case ZENITY_MSG_ERROR:
-      zenity_util_set_window_icon_from_stock (dialog, data->window_icon, GTK_STOCK_DIALOG_ERROR);
+      zenity_util_set_window_icon_from_icon_name (dialog, data->window_icon, "dialog-error");
       break;
       
     case ZENITY_MSG_INFO:
-      zenity_util_set_window_icon_from_stock (dialog, data->window_icon, GTK_STOCK_DIALOG_INFO);
+      zenity_util_set_window_icon_from_icon_name (dialog, data->window_icon, "dialog-information");
       break;
   
     default:
@@ -164,6 +200,7 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
       gtk_label_set_text (GTK_LABEL (text), msg_data->dialog_text);
     else 
       gtk_label_set_markup (GTK_LABEL (text), g_strcompress (msg_data->dialog_text));
+      zenity_label_widget_clipboard_selection(GTK_WIDGET (text));
   }
 
   if (msg_data->ellipsize)
@@ -207,6 +244,8 @@ zenity_msg_dialog_response (GtkWidget *widget, int response, gpointer data)
       break;
 
     default:
+      if (response < g_strv_length(zen_data->extra_label))
+        printf("%s\n",zen_data->extra_label[response]);
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
       break;
   }
